import es, playerlib, popuplib, cPickle, time, os.path

info = es.AddonInfo() 
info['name']        = "Karma" 
info['version']     = "2.0.0b" 
info['author']      = "Bonbon AKA: Bonbon367" 
info['url']         = "http://addons.eventscripts.com/addons/view/points" 
info['description'] = "A karma system where people can vote to take/give karma" 

es.ServerVar('karma_version', info['version'], 'Karma System made by Bonbon AKA Bonbon367').makepublic()

sv_options = {
	'karma_players_in_top': 10,
	'karma_delay': 10,
	'karma_perplayer_delay': 1440,
	'karma_admins': 'STEAM_0:0:11089864,STEAM_0:0:000000,STEAM_0:1:111111111',
	'karma_rankcommand_delay': 60,
	'karma_roundstart_message': 1,
}

sv = es.ServerVar
FILEPATH = es.getAddonPath('points') + '/'

### Loads ###

def load():
	for option in sv_options:
		sv(option).set(sv_options[option])

	if not es.exists('saycommand', '!karma'):
		es.regsaycmd('!karma', 'points/karmamenu_command')

	if not es.exists('saycommand', '!karmaadmin'):
		es.regsaycmd('!karmaadmin', 'points/karmaadmin_command')

	if not es.exists('saycommand', '!karmarank'):
		es.regsaycmd('!karmarank', 'points/karmarank_command')

	if not es.exists('command', 'karma_addrank'):
		es.regcmd('karma_addrank', 'points/addrank')

	karmaMenu = popuplib.easymenu('Karma', '_popup_choice', karmaMenuSelect)
	karmaMenu.addoption('Give Karma', 'Give Karma')
	karmaMenu.addoption('Take Karma', 'Take Karma')
	karmaMenu.addoption('Top', 'Top')
	karmaMenu.addoption('Rank', 'Rank')
	karmaMenu.addoption('Player Ranks', 'Player Ranks')
	karmaMenu.addoption('Commands', 'Commands')

	adminMenu = popuplib.easymenu('Karma Admin', '_popup_choice', adminMenuSelect)
	adminMenu.addoption('Give Karma', 'Give Karma')

	es.addons.registerSayFilter(karma.chatWatcher)
	es.server.cmd('es_xmexec ../addons/eventscripts/points/points.cfg')

def unload():
	if es.exists('saycommand', '!karma'):
		es.unregsaycmd('!karma')

	if es.exists('saycommand', '!karmaadmin'):
		es.unregsaycmd('!karmaadmin')

	if es.exists('saycommand', '!karmarank'):
		es.unregsaycmd('!karmarank')
	es.addons.unregisterSayFilter(karma.chatWatcher)

### Classes ###

class Karma(object):
	def __init__(self):
		self.data = self.getData()
		self.watched = {}
		self.rankNames = {}
		self.rankCommand = {}

	@staticmethod
	def getData():
		data = {}
		if os.path.isfile(FILEPATH + 'player_data.db'):
			oFile = open(FILEPATH + 'player_data.db')
			data = cPickle.load(oFile)
			oFile.close()

		for steamid in data:
			if type(data[steamid]) == int:
				data[steamid] = {'points': data[steamid], 'last give': 0, 'temp': {}, 'reasons': [], 'name': 'Unnamed'}
		return data

	def chatWatcher(self, userid, text, teamonly):
		if userid in self.watched:
			self.watched[userid]['reason'] = text.replace('\n', '').replace('\r', '').replace('"', '')
			self.watched[userid]['func'](userid, self.watched[userid])

			del self.watched[userid]
			return None, None, None
		return userid, text, teamonly

	def askReason(self, to_userid, from_userid, amount):
		steamid = es.getplayersteamid(from_userid)
		if not steamid in self.data:
			self.data[steamid] = {'points': 0, 'last give': 0, 'temp': {}, 'reasons': [], 'name': es.getplayername(from_userid)}

		if time.time() - self.data[steamid]['last give'] < int(sv('karma_delay')) * 60:
			es.tell(from_userid, '#multi', '#lightgreen[Karma]: #defaultYou have another #lightgreen%s#default before you can use karma again!'%self.timeLeft(steamid))
			return

		if not es.exists('userid', to_userid):
			es.tell(from_userid, '#multi', '#lightgreen[Karma]: #defaultPlayer has left the server!')
			return

		to_steamid = es.getplayersteamid(to_userid)
		if to_steamid in self.data[steamid]['temp']:
			seconds = int((int(sv('karma_perplayer_delay') * 60) - (time.time() - self.data['steamid']['temp'][to_steamid])))
			if seconds > 0:
				minutes, seconds = divmod(seconds, 60)
				hours, minutes = divmod(minutes, 60)

				es.tell(from_userid, '#multi', '#lightgreen[Karma]: #defaultYou have another #lightgreen%i:%02i:%02i#default before you can give that player karma!'%(hours, minutes, seconds))
				return
			del self.data[steamid]['temp'][to_steamid]

		self.watched[from_userid] = {'func': self.givePoints, 'target': [to_steamid, es.getplayername(to_userid)], 'reason': '', 'amount': amount}
		es.tell(from_userid, '#multi', '#lightgreen[Karma]: #defaultType the reason in chat!')

	def askPoints(self, userid, target):
		if not es.exists('userid', target):
			es.tell(userid, '#multi', '#lightgreen[Karma]: #defaultPlayer has left the server!')
			return

		self.watched[userid] = {'func': self.adminPoints, 'target': [es.getplayersteamid(target), es.getplayername(target)], 'reason': ''}
		es.tell(userid, '#multi', '#lightgreen[Karma]: #defaultType how many points you wish to give the player')

	def adminPoints(self, userid, data):
		target_steamid = data['target'][0]
		target_name = data['target'][1]
		amount = data['reason']

		try:
			int(amount)

		except ValueError:
			es.tell(userid, '#multi', '#lightgreen[Karma]: #defaultValue must be an integer!')
			return

		self.data[target_steamid]['points'] += int(amount)
		es.tell(userid, '#multi', '#lightgreen[Karma]: #green%s#default karma sent to #lightgreen%s'%(amount, target_name))

		target_userid = es.getuserid(target_steamid)
		if target_userid:
			es.tell(target_userid, '#multi', '#lightgreen[Karma]: #defaultAn admin has given you #lightgreen%s#default karma'%amount)
		self.saveData()

	def givePoints(self, from_userid, data):
		to_steamid = data['target'][0]
		to_name = data['target'][1]
		from_steamid = es.getplayersteamid(from_userid)

		reason = data['reason'][:128]
		amount = data['amount']

		es.tell(from_userid, '#multi', '#lightgreen[Karma]: sent #green%s#default karma to #lightgreen%s'%(amount, to_name))
		to_userid = es.getuserid(to_steamid)

		if to_userid:
			es.tell(to_userid, '#multi', '#lightgreen[Karma]: #defaultYou have recieved #green%s #defaultkarma from #lightgreen%s'%(data['amount'], es.getplayername(from_userid)))
			es.tell(to_userid, '---%s'%reason)

		if not to_steamid in self.data:
			self.data[to_steamid] = {'points': 0, 'last give': 0, 'temp': {}, 'reasons': [], 'name': to_name}

		self.data[to_steamid]['points'] += 1
		self.data[to_steamid]['reasons'].insert(0, reason)

		self.data[from_steamid]['last give'] = time.time()
		self.data[from_steamid]['temp'][to_steamid] = time.time()
		self.saveData()

	def timeLeft(self, steamid):
		seconds = int((int(sv('karma_delay')) * 60) - (time.time() - self.data['steamid']['last give']))
		minutes, seconds = divmod(seconds, 60)
		hours, minutes = divmod(minutes, 60)
		return '%i:%02i:%02i'%(hours, minutes, seconds)

	def getRank(self, steamid):
		sorted_ranks = sorted(self.data, key=lambda x: self.data[x]['points'], reverse=True)
		return sorted_ranks.index(steamid) + 1, len(sorted_ranks)

	def getRankName(self, steamid):
		name = 'Phail'
		amount = self.data[steamid]['points']

		for i in sorted(self.rankNames):
			if amount < i:
				break
			name = self.rankNames[i]
		return name

	def saveData(self):
		iFile = open(FILEPATH + 'player_data.db', 'w')
		cPickle.dump(self.data, iFile)
		iFile.close()

karma = Karma()

### Commands ###

def karmamenu_command():
	popuplib.send('Karma', es.getcmduserid())

def karmaadmin_command():
	userid = es.getcmduserid()
	if not es.getplayersteamid(userid) in str(sv('karma_admins')).replace(' ', '').replace('"', '').split(','):
		es.tell(userid, '#multi', '#lightgreen[Karma]: #defaultYou must be an admin to use this feature!')
		return
	popuplib.send('Karma Admin', userid)

def addrank():
	args = es.getargs()
	if not args:
		return

	args = args.replace('\n', '').replace('\r', '').split(' ')
	if len(args) < 2:
		return

	try:
		karma.rankNames[int(args[0])] = ' '.join(args[1:])

	except ValueError:
		pass

def karmarank_command():
	userid = es.getcmduserid()
	steamid = es.getplayersteamid(userid)

	if not int(sv('karma_rankcommand_delay')):
		es.tell(userid, '#multi', '#lightgreen[Karma]: #defaultthis command is disabled')
		return

	if not userid in karma.rankCommand:
		karma.rankCommand[userid] = 0

	timeLeft = int(sv('karma_rankcommand_delay')) - (time.time() - karma.rankCommand[userid])
	if timeLeft > 0:
		es.tell(userid, '#multi', '#lightgreen[Karma]: #defaultYou must wait another #green%i#default seconds before using this command again'%timeLeft)
		return

	if not steamid in karma.data:
		es.tell(userid, '#multi', '#lightgreen[Karma]: #defaultYou have not given or sent karma!')
		return

	rankpos, ranktot = karma.getRank(steamid)
	rankname = karma.getRankName(steamid)

	karma.rankCommand[userid] = time.time()
	es.tell(userid, '#multi', '#lightgreen[Karma]: %s#default is #lightgreen%s #green[%s/%s]'%(es.getplayername(userid), rankname, rankpos, ranktot))

### Popups ###

def karmaMenuSelect(userid, choice, popupid):
	if choice in ['Give Karma', 'Take Karma']:
		choosePlayerMenu = popuplib.easymenu('Karma Player Select', '_popup_choice', choosePlayerMenuSelect)
		for userid2 in es.getUseridList():
			if userid == userid2:
				continue
			choosePlayerMenu.addoption([userid2, -1 if choice == 'Take Karma' else 1], es.getplayername(userid2))
		popuplib.send('Karma Player Select', userid)

	elif choice == 'Rank':
		steamid = es.getplayersteamid(userid)
		if not steamid in karma.data:
			es.tell(userid, '#multi', '#lightgreen[Karma]: #defaultYou have not given or recieved karma!')
			return

		rankMenu = popuplib.easymenu('Karma Rank', '_popup_choice', lambda *a: None)
		rankMenu.addoption(None, 'Rank: [%s/%s]'%karma.getRank(steamid))
		rankMenu.addoption(None, 'Rank Name: %s'%karma.getRankName(steamid))
		rankMenu.addoption(None, 'Points: %s'%karma.data[steamid]['points'])

		for reason in karma.data[steamid]['reasons'][:5]:
			rankMenu.addoption(None, reason)
		popuplib.send('Karma Rank', userid)

	elif choice == 'Top':
		topMenu = popuplib.easymenu('Karma Top', '_popup_choice', lambda *a: es.tell(a[0], '#multi', '#lightgreen[Karma]: %s#default is #lightgreen%s'%(a[1][0], a[1][1])))
		sorted_ranks = sorted(karma.data, key=lambda x: karma.data[x]['points'], reverse=True)

		for i in range(min(len(sorted_ranks), int(sv('karma_players_in_top')))):
			data = karma.data[sorted_ranks[i]]
			topMenu.addoption([karma.data[sorted_ranks[i]]['name'], karma.getRankName(sorted_ranks[i])], '%s - %s'%(data['name'], data['points']))
		popuplib.send('Karma Top', userid)

	elif choice == 'Player Ranks':
		chooseOnlinePlayerMenu = popuplib.easymenu('Choose Online Player', '_popup_choice', chooseOnlinePlayerMenuSelect)
		for userid2 in es.getUseridList():
			chooseOnlinePlayerMenu.addoption([es.getplayername(userid2), es.getplayersteamid(userid2)], es.getplayername(userid2))
		popuplib.send('Choose Online Player', userid)

	elif choice == 'Commands':
		es.tell(userid, '#multi', '#lightgreen[Karma]: #defaultCommands are #green!karmarank, !karma' + (', !karmaadmin' if es.getplayersteamid(userid) in str(sv('karma_admins')).replace(' ', '').replace('"', '').replace('\n', '').replace('\r', '').split(',') else ''))

def chooseOnlinePlayerMenuSelect(userid, choice, popupid):
	target_steamid = choice[1]
	target_name = choice[0]

	if not target_steamid in karma.data:
		es.tell(userid, '#multi', '#lightgreen[Karma]: %s#default has not given or recieved karma!'%target_name)
		return

	rankpos, ranktot = karma.getRank(target_steamid)
	rankname = karma.getRankName(target_steamid)
	es.tell(userid, '#multi', '#lightgreen[Karma]: %s#default is #lightgreen%s #green[%s/%s]'%(target_name, rankname, rankpos, ranktot))

def adminMenuSelect(userid, choice, popupid):
	chooseAdminPlayerMenu = popuplib.easymenu('Karma Admin Choose Player', '_popup_choice', chooseAdminPlayerMenuSelect)
	for userid2 in es.getUseridList():
		chooseAdminPlayerMenu.addoption(userid2, es.getplayername(userid2))
	popuplib.send('Karma Admin Choose Player', userid)

def chooseAdminPlayerMenuSelect(userid, choice, popupid):
	karma.askPoints(userid, choice)

def choosePlayerMenuSelect(userid, choice, popupid):
	karma.askReason(choice[0], userid, choice[1])

### Events ###

def player_disconnect(ev):
	userid = int(ev['userid'])
	if userid in karma.watched:
		del karma.watched[userid]

	if userid in karma.rankCommand:
		del karma.rankCommand[userid]

def player_activate(ev):
	steamid = ev['es_steamid']
	if steamid in karma.data:
		karma.data[steamid]['name'] = ev['es_username']

def player_changename(ev):
	steamid = ev['es_steamid']
	if steamid in karma.data:
		karma.data[steamid]['name'] = ev['newname']

def round_start(ev):
	if int(sv('karma_roundstart_message')):
		es.msg('#multi', '#lightgreen[Karma]: #defaultKarma is active, type #green!karma#default for more info')
